/*
 * UDP Logger, based on the iplogger package (specifically tcplog.c) by:
 *
 *	IpLogger Package
 *	Mike Edulla
 *	medulla@infosoc.com
 *
 * Modified by Psionic Alpha
 * psionic@darklight.com
 * Tue Jan 13 00:12:54 EST 1998
 * 
 * NOTES: (1) The define NAMESERVERIP is used to filter out all requests that
 *        go to whatever your most common name server is, as they are truely
 *        annoying and really there is no need to log them.  If you have need,
 *        just change it to something silly like "DONTSCANME", or anything else,
 *        that isn't an IP address, because its only doing a strcmp for a match.
 *
 *        (2) UDP logging is probably only for the truely paranoid.  There is 
 *        udp packets that are always flying around, because as far as I can 
 *        tell, the UDP protocol isn't streamed like TCP.  By this I mean,
 *        a TCP connection will connect on a port, and send data back and forth
 *        and eventually close, all with one connection.  A UDP connection will
 *        open a connection, send a packet, then close it.  Open a connection,
 *        send a packet, then close it.  What does this mean to you?
 *        ******ALOT****** of logging going on.
 *
 *        (3) You can NOT do a gethostbyname, or anything else, to determine
 *        what the hostname of the IP that made the connection.  Why? because
 *        the nameservice uses UDP packets.  When you go to look up the IP,
 *        the name server will send it back on a UDP packet, you'll log that,
 *        go to look up the IP of the name server that sent the UDP packet,
 *        etc, etc, and get into an infinite loop with your name server. BAD!
 *     
 *        (4) I've set the default openlog facility to LOG_LOCAL3, as I use
 *        uselog to dump everything from local3 into its own log file.  You
 *        can change this to whatever you like, or change it back to LOG_DAEMON,
 *        to have it sent to your default messages file.
 *
 *        (5) I have ONLY tried this on a linux 2.0.33 box, and while this
 *        should be portable to any OS that the original iplogger package
 *        was, I have no way of verifying it.
 *       
 *        (6) I attempted to contact the original author of the tcplog.c
 *        package, but the E-Mail address was no longer valid. 
 *
 *	  (7) I'm extremely busy, I made this for myself and am only releasing
 *        it because I havn't seen any other UDP loggers around.  I will not
 *        provide tech support, or anything else of that nature.  If you find
 *        a serious bug, you can e-mail me at the above address, but otherwise
 *        I probably won't respond.  (And I definately won't be adding anything
 *        else to it).
 * 
 */
 

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <linux/ip.h>
#include <linux/udp.h>
extern int errno;

#ifndef NOFILE
#define NOFILE 1024
#endif

// We will filter the name server requests
#define NAMESERVERIP "24.92.32.104"

int go_background(void);
char *hostlookup(unsigned long int);
char *servlookup(unsigned short);

struct ippkt {
   struct iphdr ip;
   struct udphdr udp;
   char buffer[10000];
} pkt;

int go_background(void) {
   int fd;
   int fs;
   
   if(getppid()!=1) {
      signal(SIGTTOU, SIG_IGN);
      signal(SIGTTIN, SIG_IGN);
      signal(SIGTSTP, SIG_IGN);
      fs=fork();
      if(fs<0) {
         perror("fork");
         exit(1);
      }
      if(fs>0) exit(0);
      setpgrp();
      fd=open("/dev/tty", O_RDWR);
      if(fd>=0) {
         ioctl(fd, TIOCNOTTY, (char *)NULL);
         close(fd);
      }
   }
   for(fd=0;fd<NOFILE;fd++) close(fd);
   errno=0;
   chdir("/");
   umask(0);
}
               
int main(void) {
   int s;
   int i;
   char *temp;

   setuid(0);   
   if(geteuid()!=0) {
      printf("This program requires root privledges\n");
      exit(0);
   }
   go_background();
   // last param in this is the socket type
   // originally was 6, for the tcp protocol
   // setting it to 17 for the udp protocol
   s=socket(AF_INET,SOCK_RAW,17);
   openlog("udplog",0,LOG_LOCAL3);
   
   while(1) {
      read(s,(struct ippkt *)&pkt,9999);
      // Log everything we get but the name server
      temp=hostlookup(pkt.ip.saddr);
      if(temp!=NULL) syslog(LOG_NOTICE,"%s connection attempt from %s",servlookup(pkt.udp.dest),temp);
   }
}         

char *hostlookup(unsigned long int in) {
   static char blah[1024];
   struct in_addr i;
   struct hostent *he;
         
   i.s_addr=in;
   // This use to be a gethostbyname, but name servers use udp
   // which causes an infinite, and really slow, loop
   strcpy(blah,inet_ntoa(i));
   // if its the same as the name server, return null
   // and then check for null before logging
   if(strcmp(blah,NAMESERVERIP)==0) return NULL;
   return blah;
}
                        

char *servlookup(unsigned short port) {
   struct servent *se;
   static char buff[1024];
   
   se=getservbyport(port, "udp");
   if(se==NULL) sprintf(buff, "port %d",ntohs(port));
   else sprintf(buff,"%s",se->s_name);
   return buff;
}

